/*
 * @(#)SerialReader.java	1.12 98/06/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Modifyed by Orit Shaer
 */
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;

import javax.comm.SerialPort;
import javax.comm.SerialPortEvent;
import javax.comm.SerialPortEventListener;
import javax.comm.CommPortIdentifier;

import java.awt.Dimension;
import java.awt.event.*;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JProgressBar;

public class SerialReader extends JPanel
   implements Runnable, SerialPortEventListener, ActionListener
{

   private JFrame frame;
   private JProgressBar light;
   private JProgressBar touch;


   
   private InputStream inputStream;
   private OutputStream outputStream;
   private int opcode;
   

   
   public SerialReader() {
      super();
      setOpaque(true);
      setPreferredSize(new Dimension(200, 300));

      JButton starta = new JButton("Start Output A");
      starta.setActionCommand("starta");
      starta.addActionListener(this);

      JButton startb = new JButton("Start Output B");
      startb.setActionCommand("startb");
      startb.addActionListener(this);

      light = new JProgressBar(0, 255);
      light.setStringPainted(true);
      light.setString("Light Sensor");

      touch = new JProgressBar(0, 1);
      touch.setStringPainted(true);
      touch.setString("Switch");
      

      this.add(starta);
      this.add(startb);
      this.add(light);
      this.add(touch);
      
      this.frame = new JFrame("HandyBoard Demo");
      this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      this.frame.setContentPane(this);
      this.frame.pack();
      this.frame.setVisible(true);

      this.opcode = '\0';
      
      try {
         CommPortIdentifier portId =
         CommPortIdentifier.getPortIdentifier("COM1");
         SerialPort port = (SerialPort)portId.open("COMP150 TUI",
                                                   2000);

         this.inputStream = port.getInputStream();
         this.outputStream = port.getOutputStream();
         
         System.out.println("Opened connection to COM1");
         port.addEventListener(this);
         port.notifyOnDataAvailable(true);
         port.setSerialPortParams(9600,
                                  SerialPort.DATABITS_8,
                                  SerialPort.STOPBITS_1,
                                  SerialPort.PARITY_NONE);

         Thread reader = new Thread(this);
         reader.start();
      }
      catch (Exception e) {
         System.err.println(e);
      }
   }

   
   public void actionPerformed(ActionEvent e) {
      String command = e.getActionCommand();
      try { 
         if ("starta".equals(command)) {
            this.outputStream.write(0);
         }
         
         else if ("startb".equals(command)) {
            this.outputStream.write(1);
         }
      }
      catch (IOException iox) {
         System.err.println(iox);
      }
   }

   
   public void serialEvent(SerialPortEvent event) {

      if (event.getEventType() == SerialPortEvent.DATA_AVAILABLE) {

         try {

            int data = 0;
            while (inputStream.available() > 0) {
               data = inputStream.read();
            }

            if (opcode == '\0') {
               opcode = (char)data;
            }

            //---------------------------------------
            // Obtain light sensor reading
            //---------------------------------------
            else if (opcode == 'L') {
               this.light.setValue(255 - data);
               this.light.setString("Light: " + (255 - data));
               repaint();
               opcode = '\0';
            }

            //---------------------------------------
            // Obtain switch reading
            //---------------------------------------
            else if (opcode == 'S') {
               if (data == 0) {
                  this.touch.setValue(1);
                  this.touch.setString("Switch: ON");
               } else {
                  this.touch.setValue(0);
                  this.touch.setString("Switch: OFF");
               }
               repaint();
               opcode = '\0';
            }
            
         }
         catch (IOException iox) {
            System.err.println(iox);
         }
      }         
   }

   
   public void run() {
      while (true) {
         try { Thread.sleep(20000); }
         catch (InterruptedException e) { ; }
      }
   }


   /**
    * Create the GUI and show it.  For thread safety,
    * this method should be invoked from the
    * event-dispatching thread.
    */
   private static void createAndShowGUI() {
      SerialReader sr = new SerialReader();
   }

   
   public static void main(String [] args) {

      //Schedule a job for the event-dispatching thread:
      //creating and showing this application's GUI.
      javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
               createAndShowGUI();
            }
         });
   }
}
